import { Service } from "./service";
export class RouteService extends Service {
    constructor() {
        super(...arguments);
        this.routes = [];
        this.prefix = "/";
    }
    addRoute(RouteHandler) {
        this.routes;
    }
    install(server) {
        for (let route of this.routes) {
            let path = this.prefix + route.path;
            let method = route.method.toLowerCase();
            server[method](path, (req, res, next) => {
                route.handleRawRequest(req, res, next);
            });
        }
    }
}
export class Route {
    constructor(service) {
        this.service = service;
        this.method = "GET";
        this.params = {};
        this.body = {};
        this.query = {};
        let name;
        Object.defineProperty(this, "name", {
            get: () => {
                if (name)
                    return name;
                return `${this.method}_${this.path}`;
            },
            set: (value) => {
                name = value;
            }
        });
    }
    handleRawRequest(req, res, next) {
        this.req = req;
        this.res = res;
        this.next = next;
        this.body = req.body || {};
        this.params = req.params || {};
        this.query = req.query || {};
        this.handle();
    }
    end(content) {
        this.res.contentType(this.mime);
        this.res.end(content);
    }
}
export class DynamicResourceRoute extends Route {
}
export class PageRoute extends Route {
    html(content) {
        this.end(content);
    }
}
export class APIRoute extends Route {
    success(data) {
        this.json({
            state: true,
            data
        });
    }
    error(err) {
        this.json({
            state: false,
            error: err.name,
            message: err.message
        });
    }
    json(data) {
        this.res.json(data);
    }
}
