import { DependencyManager } from "./dependency";
import * as async from "async";
import * as fs from "fs";
import * as pathModule from "path";
export class ServiceContext {
    constructor() {
        this.services = {};
        this.dependencyManager = new DependencyManager;
        this.supportedLoadExtnames = [".ts", ".js"];
    }
    register(s) {
        this.dependencyManager.add(s.name, s.dependencies || null);
        this.services[s.name] = s;
    }
    loadServices(path) {
        path = pathModule.resolve(path);
        let files = fs.readdirSync(path);
        files = files.filter((item) => {
            let ext = pathModule.extname(item);
            if (this.supportedLoadExtnames.indexOf(ext) < 0) {
                return false;
            }
            return true;
        });
        files = files.map((item) => {
            let ext = pathModule.extname(item);
            let base = pathModule.basename(item, ext);
            return pathModule.join(path, base);
        });
        let toLoads = [];
        for (let file of files) {
            if (toLoads.indexOf(file) >= 0)
                continue;
            toLoads.push(file);
        }
        for (let file in toLoads) {
            this.loadService(file);
        }
    }
    loadService(path) {
        console.log(`Load service ${path}`);
        let result = require(path) || {};
        for (let name in result) {
            let obj = result[name];
            if (obj instanceof Service) {
                this.register(obj);
            }
        }
    }
    setup(finish = () => null) {
        let initialSequence = this.dependencyManager.resolve();
        let names = Object.keys(this.services);
        async.eachSeries(names, (name, done) => {
            let service = this.services[name];
            if (service.initialize) {
                if (service.initialize.length == 2) {
                    service.initialize(this.services, (err) => {
                        if (err) {
                            console.error(`Fail to initialize service ${name}.`);
                            done(err);
                            return;
                        }
                        console.log(`Service ${name} initialized`);
                        done();
                    });
                }
                else {
                    service.initialize(this.services);
                    console.log(`Service ${name} initialized`);
                    done();
                }
            }
            else {
                console.log(`Service ${name} initialized`);
                done();
            }
        }, (err) => {
            if (err) {
                finish(err);
                return;
            }
            console.log("All service initialized");
        });
    }
}
export class Service {
    constructor() {
        this.dependencies = [];
    }
}
